/*
   --------------------------------------------------------
   Filename.....: applicationIni.cc
   ClassName....: applicationIni
   Purpose......: Create and use an ini file
                  compatible with Windows security
                  model and standard User privileges.
   Programmer...: Rick Miller -  www.treturn.com/dbase
   Date.........: February 18, 2011
   Notes........: See Notes.
   Written for..: dB2K 0.1 and newer.
   Rev. History.: None.
   Dependencies.: None.
   Usage........: set procedure to
                  applicationIni.cc additive
   HelpFile.....: None.
   Example:.....: See Example.
   Sample Form..: applicationIni.wfm
   --------------------------------------------------------
   Example:
   --------------------------------------------------------
      oRef  =  new applicationIni()
      ?  oRef.value
      if oRef.fileCheck()
         // if there is an ini file for the current exe
         //    that is running.
         //    if type("_app.useUACPaths") == "L"
         //       1) nothing to do, Plus 2.70 is in use.
         //    else
         //       1) a Windows 7 compatible folder is
         //          created when it does NOT already exist.
         //       2) the application.ini file is copied to
         //          the compatible folder when it
         //          does NOT already exist.
         //    endif
         // endif
      endif
      oRef.release()
      oRef  := null
   --------------------------------------------------------
   Notes:
   --------------------------------------------------------
      1) any new ini file created is for programmer use.
      2) when Plus 2.70 or newer IDE or Runtime
          is used, _app.iniFile is used by default.
   --------------------------------------------------------
   Properties:          type:       default:
   --------------------------------------------------------
      value             <char>      CSIDL_LOCAL_APPDATA\
                                    appFolder.
                                    Similar to Plus 2.70.
   --------------------------------------------------------
   Methods:
   --------------------------------------------------------
      close()     call to release,
                  close this procedure file.
      fileCheck([<char>[,<char>[,<char]]])
                  use to automatically check for an
                     application.ini file and copy it to
                     a proper folder with full permissions
                     when it does NOT exist.
                  can be used to specify the ini file to
                     copy with parameter2 (1st choice)
                     and/or parameter3 (2nd choice).
         return true/false (ini file exists).
      getFilename([<logical>[,<logical>]])
         return an ini filename.
      getKeynames(<char> section[, <char> ini filename])
         return an array of key names.
      getPath([<char> ini filename])
         return the path.
      getSections([<char> ini filename])
         return an array of section names.
      getValue(<char> Section,;
               <char> Keyname[, <char> ini filename])
         return a string value.
      release()   release object.
      setValue(<char> Section, <char> Keyname,;
               <char/int> Value[, <char> ini filename])
                  write a string value.
         return true/false (write took place).
   --------------------------------------------------------
   Revisions:
   --------------------------------------------------------
*/
// <user name>\Local Settings\Application Data (non roaming)
#define  CSIDL_LOCAL_APPDATA              0x001c
// All Users\Application Data
#define  CSIDL_COMMON_APPDATA             0x0023
// C:\Program Files
#define  CSIDL_PROGRAM_FILES              0x0026
#define  MAX_LINELENGTH                   0x0800
#define  _APPLICATIONINI_CLASSNAME_       "appliactionIni"
#define  _APPLICATIONINI_PUBLISHERINFO_   "Rick Miller"
#define  _APPLICATIONINI_VERSIONINFO_     "11.0218"
//---------------------------------------------------------------//
//          start of constructor.
//---------------------------------------------------------------//
class applicationIni
   protect  x_GetFilenameAddon,;
            x_GetSpecialFolder,;
            x_GetSpecialFolderIni,;
            x_InitializeExterns,;
            x_ValidateIniFolder
   class::x_InitializeExterns()
   this.className    =  _APPLICATIONINI_CLASSNAME_
   this.publishInfo  =  _APPLICATIONINI_PUBLISHERINFO_
   this.versionInfo  =  _APPLICATIONINI_VERSIONINFO_
   this.value        =  this.getFilename()
   //------------------------------------------------------------//
   //       end of constructor   -  methods below.
   //------------------------------------------------------------//
   // release object, close this procedure file.
   function close
      this.release()
      try
         close procedure (program(1))
      catch(exception e)
      endtry
   return
   //------------------------------------------------------------//
   // parameters (none are required).
   //    cNewIni  <char>   the ini file to create.
   //    cOldIni  <char>   the app.exe ini filename to copy.
   //    cComIni  <char>   the ALLUSERS ini filename to copy.
   // returns true/false  does the cNewIni file exist ?
   function fileCheck(cNewIni, cOldIni, cComIni)
      Local bRet
      bRet  =  false
      if not type("argvector(1)") == "C"
         // get an ini filename in CSIDL_LOCAL_APPDATA folder.
         // when type("_app.useUACPaths") == "L"
         //    use the dBASE Plus _app.iniFile property.
         cNewIni  := this.getFilename()
      endif
      if not type("argvector(2)") == "C"
         // get an ini filename in application folder.
         cOldIni  := this.getFilename(true)
      endif
      if not type("argvector(3)") == "C"
         // get an ini filename in CSIDL_COMMON_APPDATA folder.
         cComIni  := this.getFilename(false, true)
      endif
      if file(cNewIni)
         // cNewIni file already exists.
         bRet  := true
      elseif file(cOldIni)
         // cOldIni file does exist.
         // cNewIni file does not exist.
         if this.x_ValidateIniFolder(cNewIni)
            bRet  := RMM_CopyFile(;
                     cOldIni,;
                     cNewIni,;
                     false)
         endif
      elseif file(cComIni)
         // cComIni file does exist.
         // cNewIni file does not exist.
         if this.x_ValidateIniFolder(cNewIni)
            bRet  := RMM_CopyFile(;
                     cComIni,;
                     cNewIni,;
                     false)
         endif
      endif
   return   iif(bRet, true, false)
   //------------------------------------------------------------//
   // default return value is CSIDL_LOCAL_APPDATA ini filename.
   // parameters.
   //    bWindows98 <logical> when true
   //       return the application folder ini filename.
   //    bCommon <logical> when true
   //       return the CSIDL_COMMON_APPDATA folder ini filename.
   // Note: when both parameters are true, bWindows98 is used.
   function getFilename(bWindows98, bCommon)
      Local cFile
      if type("_app.useUACPaths") == "L"
         // Plus 2.70 or newer is in use.
         // no need to do any more.
         cFile =  "" + _app.iniFile
      else
         // Plus 2.62 or older.
         // get the application.ini filename.
         if type("_app.exeName") == "C"
            cFile =  _app.exeName.left(;
                     _app.exeName.lastIndexOf(".")) +;
                     ".ini"
         else
            // api to get the application.ini filename.
            Local cValu, nLen
            cValu =  replicate(chr(0), 260)
            nLen  =  RMM_GetModuleFileName(0, cValu, 260)
            cFile =  cValu.left(nLen)
            cFile := cFile.left(;
                     cFile.lastIndexOf(".")) +;
                     ".ini"
         endif
         if bWindows98
            // return the application folder ini filename.
         elseif bCommon
            // return the CSIDL_COMMON_APPDATA folder ini filename.
            cFile := this.x_GetSpecialFolderIni(;
                     cFile,;
                     CSIDL_COMMON_APPDATA)
         else  // default.
            // this is the default for Plus versions before 2.70.
            // return the CSIDL_LOCAL_APPDATA folder ini filename.
            cFile := this.x_GetSpecialFolderIni(;
                     cFile,;
                     CSIDL_LOCAL_APPDATA)
         endif
      endif
   return   cFile
   //------------------------------------------------------------//
   // return an array of key names.
   function getKeynames(cSection, cIni)
      Local aRet, cRet, cName, nChr, nLen, nSize
      if not type("argvector(2)") == "C"
         cIni  := "" + this.value
      endif
      aRet  =  new array()
      nLen  =  -2
      nSize =  0
      do while nLen == (nSize - 2)
         nSize += MAX_LINELENGTH
         cRet  =  replicate(chr(0), nSize)
         nLen  := RMM_GetIniKeyNames(cSection, cRet, nSize, cIni)
      enddo
      cName =  ""
      for i = 0 to (nLen - 1)
         nChr  =  cRet.getByte(i)
         if nChr == 0
            aRet.add(cName.left(cName.indexOf("=")))
            cName := ""
         else
            cName += chr(nChr)
         endif
      endfor
   return   aRet
   //------------------------------------------------------------//
   // return the path of the passed ini filename or this.value.
   function getPath(cIni)
      if not type("argvector(1)") == "C"
         cIni  := "" + this.value
      endif
   return   cIni.left(cIni.lastIndexOf("\"))
   //------------------------------------------------------------//
   // return an array of section names.
   function getSections(cIni)
      Local aRet, cRet, cSect, nChr, nLen, nSize
      if not type("argvector(1)") == "C"
         cIni  := "" + this.value
      endif
      aRet  =  new array()
      nSize =  MAX_LINELENGTH
      cRet  =  replicate(chr(0), MAX_LINELENGTH)
      nLen  =  RMM_GetIniSections(cRet, nSize, cIni)
      cSect =  ""
      for i = 0 to (nLen - 1)
         nChr  =  cRet.getByte(i)
         if nChr == 0
            aRet.add(cSect)
            cSect := ""
         else
            cSect += chr(nChr)
         endif
      endfor
   return   aRet
   //------------------------------------------------------------//
   // return a string value.
   function getValue(cSection, cName, cFile)
      Local cRet, cValu, nLen, nSize
      cRet  =  ""
      if not type("argvector(3)") == "C"
         cFile := "" + this.value
      endif
      nSize =  MAX_LINELENGTH
      cValu =  space(MAX_LINELENGTH)
      // returns number of characters copied to the buffer,
      // not including the terminating null character.
      nLen  =  RMM_GetIniString( ;
               "" + cSection,;
               "" + cName,;
               "",;
               cValu,;
               nSize,;
               cFile)
      if nLen > 0
         cRet  := cValu.left(nLen)
      endif
   return   cRet
   //------------------------------------------------------------//
   // display this object in the inspector.
   function inspect
      inspect(this)
   return
   //------------------------------------------------------------//
   function release
      try
         release object this
      catch(exception e)
      endtry
   return
   //------------------------------------------------------------//
   // write a string value.
   function setValue(cSection, cName, pValu, cFile)
      if not type("argvector(4)") == "C"
         cFile := "" + this.value
      endif
   return   RMM_WriteIniString(;
            "" + cSection,;
            "" + cName,;
            "" + pValu,;
            cFile)
   //------------------------------------------------------------//
   function x_GetFilenameAddon(cTest, cUser, cPath, cProg)
      Local cRet
      if cTest.left(cUser.length) == cUser.toLowerCase()
         cRet  =  cPath.substring(cUser.length, cPath.length)
      elseif cTest.left(cProg.length) == cProg.toLowerCase()
         cRet  =  cPath.substring(cProg.length, cPath.length)
      elseif cTest.left(2) == "\\"
         cRet  =  cPath.substring(1, cPath.length)
      else
         cRet  =  cPath.subString(;
                  cPath.indexOf("\", 3),;
                  cPath.length)
      endif
   return   "" + cRet
   //------------------------------------------------------------//
   function x_GetSpecialFolder(nFold)
      Local cRet
      cRet  =  space(260)
      RMM_SHGetFolderPath(null, nFold, 0, null, cRet)
   return   "" + cRet
   //------------------------------------------------------------//
   // return an ini filename for a special folder.
   // parameters.
   //    cFile <char>   filename the return value is based upon.
   //    nFolder <int>  special folder used in the return value.
   function x_GetSpecialFolderIni(cFile, nFolder)
      Local cIni, cPath, cProg, cTest, cUser
      cIni  =  cFile.substring(;
               cFile.lastIndexOf("\") + 1,;
               cFile.lastIndexOf("."))
      cPath =  cFile.left(cFile.lastIndexOf("\") + 1)
      cUser =  this.x_GetSpecialFolder(nFolder)
      cProg =  this.x_GetSpecialFolder(CSIDL_PROGRAM_FILES)
      cTest =  cPath.toLowerCase()
      cAdd  =  this.x_GetFilenameAddon(cTest, cUser, cPath, cProg)
   return   (cUser + cAdd + cIni + ".ini")
   //------------------------------------------------------------//
   function x_InitializeExterns
      if not type("RMM_CopyFile") =="FP"
         extern   CLOGICAL ;
                  RMM_CopyFile(;
                  CSTRING,;
                  CSTRING,;
                  CLOGICAL) ;
                  kernel32 from "CopyFileA"
      endif
      if not type("RMM_CreateDirectory") == "FP"
         extern   CLOGICAL ;
                  RMM_CreateDirectory(;
                  CSTRING,;
                  CPTR) ;
                  kernel32 from "CreateDirectoryA"
      endif
      if not type("RMM_GetIniKeyNames") == "FP"
         extern   CULONG ;
                  RMM_GetIniKeyNames(;
                  CSTRING,;
                  CPTR,;
                  CULONG,;
                  CSTRING) ;
                  kernel32 from "GetPrivateProfileSectionA"
      endif
      if not type("RMM_GetIniSections") == "FP"
         extern   CULONG ;
                  RMM_GetIniSections(;
                  CPTR,;
                  CULONG,;
                  CSTRING) ;
                  kernel32 from "GetPrivateProfileSectionNamesA"
      endif
      if not type("RMM_GetIniString") == "FP"
         extern   CULONG ;
                  RMM_GetIniString( ;
                  CSTRING,;
                  CSTRING,;
                  CSTRING,;
                  CSTRING,;
                  CULONG,;
                  CSTRING) ;
                  kernel32 from "GetPrivateProfileStringA"
      endif
      if not type("RMM_GetModuleFileName") == "FP"
         extern   CINT ;
                  RMM_GetModuleFileName(;
                  CHANDLE,;
                  CSTRING,;
                  CINT) ;
                  kernel32 from "GetModuleFileNameA"
      endif
      if not type("RMM_SHGetFolderPath") == "FP"
         extern   CLONG ;
                  RMM_SHGetFolderPath( ;
                  CHANDLE,;
                  CINT,;
                  CHANDLE,;
                  CULONG,;
                  CSTRING) ;
                  shfolder from "SHGetFolderPathA"
      endif
      if not type("RMM_WriteIniString") == "FP"
         extern   CLOGICAL ;
                  RMM_WriteIniString( ;
                  CSTRING,;
                  CSTRING,;
                  CSTRING,;
                  CSTRING) ;
                  kernel32 from "WritePrivateProfileStringA"
      endif
   return
   //------------------------------------------------------------//
   // return true/false.
   // parameters.
   //    cIni  <char>   Windows 7 compatible ini filename
   //                      to validate folder for.
   function x_ValidateIniFolder(cIni)
      Local cPath, cTest, cUser, nPos
      // assign a dummy value in case the loop gets missed.
      cPath =  "dgsjuetyrio"
      cTest =  cIni.toLowerCase()
      cUser =  space(260)
      // fill cUser with whatever CSIDL_LOCAL_APPDATA is.
      RMM_SHGetFolderPath(;
               null,;
               CSIDL_LOCAL_APPDATA,;
               0,;
               null,;
               cUser)
      // set the starting folder to check.
      if cTest.left(cUser.length) == cUser.toLowerCase()
         // begin with a sub-folder of CSIDL_LOCAL_APPDATA.
      elseif cTest.left(2) == "\\"
         // begin with a network share name (\\server\share).
         // network folder (not windows 7 compatible).
         nPos  =  cIni.left(cIni.indexOf("\", 3))
         cUser := cIni.left(cIni.indexOf("\", nPos + 1))
      else
         // begin with a local folder (c:\folder).
         // local folder (not windows 7 compatible).
         cUser := cIni.left(cIni.indexOf("\", 3))
      endif
      nPos  =  cIni.indexOf("\", cUser.length + 1)
      do while not nPos == -1
         // loop thru the path,
         // creating the directory if necessary.
         cPath := cIni.left(nPos)
         if empty(funique(cPath + "\????????.tmp"))
            RMM_CreateDirectory(cPath, null)
         endif
         nPos  := cIni.indexOf("\", nPos + 1)
      enddo
   return   not empty(funique(cPath + "\????????.tmp"))
   // return success/failure.
endclass
//------------------------------------------------------------------//
//          end of class.
//------------------------------------------------------------------//

