/*
    Gradient.CC
    Gradient.cc allows painting the background color
    of any form(s) as well as the _app.FrameWin with a two color gradient.

    Authors:
       Vic McClung
       Ken Mayer (minor -- comments, formatting ...)

    History:
       July 8, 2000 -- Original (Vic McClung)
       August, 2001 -- Modifications/bug fixes (Vic McClung)

    Notes:
       The Gradient.DLL is compiled with LCCWIN32 ANSI C.  The
       function Gradient is called to paint the two color gradient.
       The Gradient.DLL can handle any number of forms background
       painting,

       If you want a solid background instead of the gradient, just
       set both colors the same.

    Usage:
       For an application framewin:

       set procedure to gradient.cc addi
       _app.gradient = new gradient()
       _app.SpeedBar = false
       _app.StatusBar = false
       shell(false, false)
      // pass parameters for colors/direction?
      _app.gradient.Color1= _app.gradient.rgb(0,0,255)
      _app.gradient.Color2 = _app.gradient.rgb(255,0,0)
      _app.gradient.dir = 1   // vertical, if you want
                              // horizontal, use any other number
      _app.gradient.Open()

      // to allow the user to select colors:
      _app.gradient.getColor1()
      _app.gradient.getColor2()
      _app.gradient.setColors()
      // when closing the application:
      _app.gradient.Close()

      // to use with a form, instead, make a new
      // instance of the object, and pass the
      // form's hWnd as a parameter -- this example
      // assumes the form's onOpen event:
      function form_onOpen
         set procedure to gradient.cc additive
         this.gradient = new gradient( form.hwnd )
         this.gradient.Color1 = _app.gradient.rgb( 0,0,0 )
         this.gradient.Color2 = _app.gradient.rgb( 255,255,255 )
         _app.gradient.dir       = 2 // horizontal, rather than vertical
         _app.gradient.setColors() // actually paint it


   Properties:
      Color1 =    first color -- when using a direction of 1
                  (vertical), this is the color at the top,
                  when using a direction of any other numeric
                  value (horizontal), this is the color on the
                  left
      Color2 =    second color -- when using a direction of 1
                  (vertical), this is the color at the bottom,
                  when using a direction of any other numeric
                  value (horizontal), this is the color on the
                  right
      Dir     =   direction - 1 = Vertical, any other digit
                  is horizontal.
      isChild =   if a child form, not the _app.framewin
      hWnd    =   window handle

   Methods:
      Open    =   called when opening a form, or starting
                  an application if you are painting the
                  _app.framewin ...
      Close   =   should be called when closing the application
                  if you are painting the _app.framewin --
                  note it is not necessary when painting a form's
                  background.
      Repaint =   calls paint() from gradient.dll
      setColors = sets the colors using the setcolors call
                  in gradient.dll, using the class's custom
                  properties (see above).
      getColor1 = a method of calling the dBASE color
                  selector, and allowing the user to select
                  the value of the control's color1 property
      getColor2 = as above for control's color2 property
      rgb       = convert the 3 numbers returned by getColor into
                  a 32 bit ColorRef for use by Windows


   Notes:
      If no hwnd is passed to class constructor,
      the _app.FrameWin background is painted, otherwise
      form's background is painted.

      The class::repaint() method needs to be called
      when the form is resized. This is not necessary
      for the _app.framewin

   Examples:
      Run gradient.prg to see the _app.framewin get set, and
      see it run with two different forms, allowing you to change
      each ...
         OR
      Run gradient.wfm to see a single form in action ...

   Dependencies:
      gradient.dll (must be deployed with any application
                    using gradient.cc)

*/

class gradient(hwnd)
   // external declarations for gradient.dll
   extern cvoid    enable(chandle) Gradient.dll
   extern cvoid    disable(chandle) gradient.dll
   extern cvoid    setColors(chandle, clong, clong, clong) gradient.dll
   extern cvoid    paint(chandle) gradient.dll
   // get the child window handle
   extern chandle  getChildHandle(chandle, cstring) gradient.dll

   // set properties to defaults
   this.isChild = false
   this.color1  = class::rgb(255, 0, 0) // red
   this.color2  = class::rgb(0, 0, 255) // blue
   this.dir     = 1 // left to right
   // if no hWnd was passed, we assume the
   // _app.framewin
   if pcount()  == 0
      this.hwnd    = 0
   else
      this.hwnd    = hwnd
      this.isChild = true
   endif

   function Open()
      // this method must be called after the form
      // is opened (onOpen method). It is passing
      // the form.hwnd handle to the method
      // if no handle is passed use the appFrameWin
      if this.hwnd == 0
         // get the handle to the _app.FrameWin background window
         // the classname is "MDIClient"
         this.hwnd = getChildHandle(_app.FrameWin.hwnd, "MDICLIENT")
      else
         this.hwnd = getChildHandle(this.hwnd, "AFXFRAMEORVIEW40SU")
      endif
      // turn on window subclassing in gradient.dll
      enable(this.hwnd) // from gradient.dll
      class::setColors()
      class::repaint()
   return

   function repaint()
      paint(this.hwnd)
   return

   function Close()
      local nPos
      // if this is _app.FrameWin, you must call disable
      // the form.hwnd's will automatically disable themselves
      if not this.isChild
         disable(this.hwnd)
         paint(this.hwnd)
      endif
   return

   function setColors()
      // this changes the colors and repaints the
      // window
      setColors( this.hwnd, this.color1, this.color2, this.dir)
   return

   function GetColor1
      // select a color
      local cStr, r, g, b
      cStr = getcolor()
      if empty(cStr)
         return
      endif
      r = val(substr(cStr, 1, at(',', cstr, 1)-1))
      g = val(substr(cStr, at(',', cStr, 1)+1, at(',', cStr, 2)-1))
      b = val(substr(cStr, at(',', cStr, 2)+1))
      this.Color1 = class::rgb(r,g,b)
      // returns string compatible with controls colorNormal properties
   return '0x00'+itoh(class::rgb(r,g,b))

   function GetColor2
      // select the second color
      local cStr, r, g, b
      cStr = getcolor()
      if empty(cStr)
         return
      endif
      r = val(substr(cStr, 1, at(',', cstr, 1)-1))
      g = val(substr(cStr, at(',', cStr, 1)+1, at(',', cStr, 2)-1))
      b = val(substr(cStr, at(',', cStr, 2)+1))
      this.Color2 = class::rgb(r,g,b)
      // returns string compatible with controls colorNormal properties
   return '0x00'+itoh(class::rgb(r,g,b))

   function RGB(r, g, b)
      // convert the 3 numbers returned by getColor into
      // a 32 bit ColorRef for use by Windows
   return (r + g*256 + b*65536)

endclass
