/*
Class:   TreeDropper
Author:  Daniel Howard (sproket@total.net)
History: 
   1.0 Oct 31, 1998 
      ( Beta )

   1.1 Nov 07, 1998
   -  Cleaned up & tested.
   -  The treeview is now created at the form level to ensure that it
      will appear on top of all other controls no matter what the 
      zOrder of the container itself is.
   -  Added an onChange event handler.
   -  Fixed up some wierd behaviour with right clicking.
   -  Added a TreeDropper custom property to be used instead of parent 
      since the TreeView's parent is the form. 

   1.2 Dec 12, 1998
   - Got rid of the TreeDropper custom property as it is now called
     setter and is defined in dContainer.

Purpose:
   Use this object like a combobox except the dropdown is not a simple
   list but a treeview object. This is similar to the kind of object 
   used in the Vdb Inspector.

   This class requires:
   dControls.cc   - Base class definitions
   dForm.cfm      - Base form class 
   down.bmp       - Down Arrow bitmap 

   Please see dControls.cc and dForm.cfm for the specific behaviours 
   required in those classes. You can use this code in you own base 
   clases. (you DO have your own base classes right?)


   Properties:

      DropDownHeight- Specifies the dropdown height of the TreeDropper 
                      in CHAR metrics.

      ShowParents -   If this property is set to true then the value 
                      of TreeDropper with include the hierarchy.

      Seperator   -  This is the seperator between parents and children 
                     shown in the entryfield. It defaults to "."

      Selected    -  A refrence to the selected treeitem. 

Form custom properties:
   ActiveTreeDropper -  "This" will be assigned to the form by the 
                        Show method. This is so I can refrence this 
                        object from the Hide method since Hide can 
                        be called from this class or the form itself.
Other properties:

   SavedInfo         -  A simple collection object to hold overridden
                        values to be restored later.
Notes:
   -  This doesn't behave exacly like a regular combobox. The treeview 
      should disappear when you click anywhere outside the treview but 
      it doesn't unless focus moves to another object or the form is
      clicked.

   -  Be careful of the zOrder of this control. If this is control is
      under others the treedview will appear underneath them. Just use
      the layout-->bring to front menu. There are ways to handle this.
      I could create the TreeView only when needed and attach it to 
      the form. This is a little bit more complicated because you would
      normally want to fill the treeview with values before the form 
      opens but wouldn't be able to since it wouldn't exist yet!
      ( something for the next version).

   - Currently this only works with metric CHARS.


This software is donated to the public domain. You may modify 
this code in anyway you wish however the author reserves the 
right to whatever official version is published. If you have 
further ideas/bug reports/additions then contact the author and 
your name will be added to the contributors if your idea is 
implemented.

The author accepts no responsibility for the use or consequences 
of the use of this software.
*/

class TreeDropper(parentObj, name) of DCONTAINER(parentObj, name) custom

   with (this)
      left = 0.8333
      top = 4.4
      width = 49
      height = 1.8
      metric = 0        // Chars
   endwith


   this.EntryField = new TD_EntryField(this)
   with (this.EntryField)
      onLeftMouseDown = class::ShowOrHideDropDown
      height = 1
      left = 0.3333
      top = 0.05
      width = 47.1667
      value = ""
   endwith

   this.TREEVIEW = new TD_TREEVIEW(this)
   with (this.TREEVIEW)
      onLeftMouseDown = Class::TREEVIEW_onLeftMouseDown
      onRightMouseDown = Class::TREEVIEW_onRightMouseDown
      onLostFocus = Class::Hide
      height = 0.3
      left = 0.3333
      top = 1.05
      width = 47.1667
   endwith

	this.Setter.Value := ""
   this.Setter.OnChange := Class::Setter_OnChange
   this.Setter.CanChange := Class::Setter_CanChange   
   this.Setter.BeforeGetValue := Class::Setter_BeforeGetValue

   this.Expanded = false
   this.ShowParents = False
   this.Seperator = "."
   this.DropDownHeight = 7

   this.Selected = NULL

   // Storage space for temporary property values.
   this.SavedInfo = New Object()

   // Create a pushbutton at the right edge of the entryfield.
   Function Init
      this.DropBUTTON  = new TD_PUSHBUTTON(this)
      with (this.DropButton)
         onClick = class::ShowOrHideDropDown
         height = this.EntryField.Height - 0.15
         left = this.EntryField.width - 3
         top = 0.15
         width = 3
         text = ""
         value = false
      endwith
   return Super::Init()

   // If we're here than a selection has been made!
   function TREEVIEW_OnLeftMouseDown(flags, col, row)

      // Assign the selection to the container
      this.Parent.Selected := this.Selected

      // Fire off the Field's events with the assignment
      if this.Selected # NULL
         this.Parent.Setter.Value := this.Selected.Text
      else
         this.Parent.Setter.Value := ""
      endif

      // Flag expanded off
      this.Parent.Expanded := false
 
      // Hide the treeview
      this.Visible := false

      // Resize the parent container.
      this.Parent.Height = this.Parent.SavedInfo.Height

      // Kill the stupid hand cursor.
      this.mousePointer := 0
   return


   // This is fired by the entryfield or the pushbutton.
   function ShowOrHideDropDown
      if this.Parent.Expanded // Hide it 
         Class::Hide()
      else                     // Show it
         Class::Show()
      endif
   return

   // Called by the treeview's onLostFocus.
   // Can be called outside as well to manually hide the dropdown.
   Function Hide
      if type("form.ActiveTreeDropper") # "U" and form.ActiveTreeDropper # NULL
         form.ActiveTreeDropper.TreeView.Visible := false
         form.ActiveTreeDropper.Height = form.ActiveTreeDropper.SavedInfo.Height
         form.ActiveTreeDropper.Expanded := false
         form.ActiveTreeDropper.TreeView.MousePointer := 0
         form.onLeftMouseDown = form.ActiveTreeDropper.SavedInfo.OnLeftMouseDown
         form.ActiveTreeDropper = NULL
      endif
   return         

   // Can be called outside as well to manually show the dropdown.
   Function Show

      // Tell the form what the active TreeDropper is
      form.ActiveTreeDropper = this.Parent
              
      // Resize the treeview according to the dropdownheight property.
      this.Parent.SavedInfo.Height = this.Parent.Height
      this.Parent.Height = this.Parent.DropDownHeight
      this.Parent.TreeView.Width = this.Parent.Width - 1.29
      this.Parent.TreeView.Height = this.Parent.Height - 1

      // This block makes sure that all children are expanded.
      this.Parent.TreeView.Iterate({|o|; o.Expanded := true})

      // Save the oldSelected for test in OnChange
      // Not really needed since the treedropper propety handles this.
      this.Parent.SavedInfo.Selected = this.Parent.Selected

      // This block of code is to force the treeview to jump to 
      // the appropriate place in the list. Although I force a 
      // select to the 1st item in the treeview I don't assign it
      // to the TreeDropper itself. This is to ensure that if there
      // is no current selection, the treeview will display properly.
      if this.Parent.Selected = NULL
         if this.Parent.TreeView.FirstChild # NULL
            this.Parent.TreeView.FirstChild.Select()
         endif
      else
         this.Parent.Selected.Select()
      endif

      // Save the onLeftMouseClick of the form itself and attach our
      // hide method to it.
      this.Parent.SavedInfo.OnLeftMouseDown = form.onLeftMouseDown
      form.onLeftMouseDown = Class::Hide

      // Open it up
      this.Parent.TreeView.Visible = true
      this.Parent.TreeView.SetFocus()
      this.Parent.Expanded = true
   return

   // This is assigned because of some anomolous behavours of the 
   // treeview. Normally, Right clicking selects a treeitem but 
   // doesn't assign the selected property. Also Right and Left 
   // mousedown events seem to be equivilant unless an event handler
   // is defined.  So I bloody well define one here!
   function TREEVIEW_OnRightMouseDown(flags, col, row)
   return

//-------- Field Event Handlers -----//

   // You would normally override this method see the TestTreeDrop
   // form for an example of using this. As far as using this object,
   // this is pretty much all you have to deal with when using this 
   // control on your forms.
   function Setter_OnChange
      this.Owner.EntryField.Value := this.Value
   return

   // This doesn't do anything really. It's here to show you where you
   // could add validation to this object. For instance you might not
   // want a value to be assigned which is not in the array. Here is 
   // a place to handle such an event.
   function Setter_CanChange
   para xValue
   return true


   // Good old property morphing. I'm getting a real kick out of doing 
   // this. In this case I see if the treedropper wants to show the
   // parentage of the selection made. This technique is nifty and not
   // found in any other OOP language. Another reason Vdb is the best!
   function Setter_BeforeGetValue
      if this.Owner.Selected = NULL
         return ""
      endif
      // If we show the hierarchy in the selection
      if this.Owner.ShowParents
         local cText,o
         cText = this.Owner.Selected.Text
         o = this.Owner.Selected
         do while o.Parent.ClassName == "TREEITEM"
            o = o.Parent
            cText = o.Text + this.Owner.Seperator + cText
         enddo
         return ( cText )
      endif
   return ( this.Value )

endclass



Class TD_EntryField(parentObj) of dEntryField(parentObj)
protect when,MousePointer
   this.When = {; return false}
   this.MousePointer = 1
endclass

Class TD_PushButton(parentObj) of dPushButton(parentObj)
protect speedbar,upBitMap
  this.speedBar = true
  this.upBitmap = "filename Down.bmp"
endclass

Class TD_TreeView(parentObj) of dTreeView(parentObj)
protect allowEditLabels,allowEditTree,PopUpEnable

   with (this)
      visible = false
      allowEditLabels = false
      allowEditTree = false
      hasButtons = false
      hasLines = false
   endwith
   this.PopUpEnable := false         
endclass


